requirejs.config({
    paths: {
        jquery: 'CodeBlockProgrammingInteraction/runtime/js/lib/jquery.min'
    }
});

define([
    'jquery',
    'CodeBlockProgrammingInteraction/runtime/js/answerParser',
], function($, answerParser){

    const MAX = 10;

    let AnswerView = function ($form, ansList, interaction) {
        this.$form = $form;
        this.$addBtn = $form.find('.add-btn');
        this.$ansRows = $form.find('.ans-container .ans-rows');
        this.ansList = Array.isArray(ansList) ? ansList : [ansList];
        this.interaction = interaction;
    };

    AnswerView.prototype.init = function () {
        this.createAnsRows(this.ansList);
        this.addEventListeners()
        this.validAllTextarea()
        this.updateButtonState();
    }

    AnswerView.prototype.createAnsRows = function () {
        let self = this;
        if (this.ansList.length == 1){
            let $textarea = this.addAnsRow().find('textarea');
            this.setTextAreaValue($textarea, this.ansList[0]);
        } else {
            this.ansList.forEach((ansValue, idx) => {
                if(ansValue !== '') {
                    let $textarea = self.addAnsRow().find('textarea')
                    self.setTextAreaValue($textarea, ansValue);
                }
            })
        }
    }

    AnswerView.prototype.changeAnsList = function (ans) {
        let $textarea = this.getAllTextarea().last();

        if ($textarea.val() && !this.isMaxRow()){
            let $row = this.addAnsRow();
            $textarea = $row.find('textarea');
        }
        this.setTextAreaValue($textarea, ans)
        this.updateAnsList();
        this.updateButtonState();
    }

    AnswerView.prototype.setTextAreaValue = function($textarea, ansValue) {
        if(Array.isArray(ansValue)){
            $textarea.val(ansValue.join('\n'));
        } else {
            $textarea.val(ansValue);
        }
        window.setTimeout( function() {
            updateHeightTextarea($textarea)
        }, 10);
    }

    AnswerView.prototype.updateAnsList = function () {
        let self = this;
        self.ansList = [];
        self.getAllTextarea().each((idx, ele) => {
            if ($(ele).val().trim() !== ''){
                self.ansList.push($(ele).val());
            }
        })

        let length = self.ansList.length;
        if (length == 0 || (length == 1 && self.ansList[0] !== "")){
            self.ansList.unshift('');
        }

        //console.log("changeAnsList", self.ansList)
        if (this.interaction){
            this.interaction.triggerPci('changeAnsList', [self.ansList])
        }

    };

    AnswerView.prototype.getAllTextarea = function(){
        return this.$ansRows.find('textarea');
    }

    AnswerView.prototype.updateButtonState = function(){
        this.updateRemoveBtnState()
        this.updateAddBtnState()
    }

    AnswerView.prototype.updateRemoveBtnState = function(){
        if (this.$ansRows.find('textarea').length === 1) {
            this.$ansRows.find('.remove-btn').addClass('disabled');
        } else {
            this.$ansRows.find('.remove-btn').removeClass('disabled');
        }
    }

    AnswerView.prototype.updateAddBtnState = function(){
        this.$addBtn.toggleClass('disabled', this.isMaxRow());
    }

    AnswerView.prototype.isMaxRow = function(){
        return this.$ansRows.find('.ans-row').length >= MAX;
    }

    AnswerView.prototype.addAnsRow = function() {

        let $newRow = $('<div class="ans-row">\n' +
            '    <textarea name="anslist" value=""></textarea>\n' +
            '    <div class="remove-outer">\n' +
            '        <button class="btn remove-btn icon-close disabled"></button>\n' +
            '    </div>\n' +
            '    <div class="ans-row-error"><p>答えリストが不正なフォーマットです</p></div>\n' +
            '</div>');
        this.$ansRows.append($newRow);
        this.updateRemoveBtnState();
        return $newRow;
    }

    AnswerView.prototype.addEventListeners = function(){
        let self = this;

        let $ansContainer = this.$form.find('.ans-container');

        $ansContainer.find('.add-btn').click(function () {
            if (self.isMaxRow()) {
                return;
            }
            self.addAnsRow();
            self.updateButtonState();
        });

        $ansContainer.on('click', '.remove-btn', function () {
            let parentElement = this.closest('.ans-row');
            parentElement.parentNode.removeChild(parentElement);
            self.updateButtonState();
            self.updateAnsList()
        });

        $ansContainer.on('input change', 'textarea', (e) => {
            self.validTextAreaValue($(e.currentTarget))
            self.updateButtonState();
            self.updateAnsList()
            updateHeightTextarea($(e.currentTarget))
        });
    }

    AnswerView.prototype.removeEventListeners = function(){
        let $ansContainer = this.$form.find('.ans-container');
        $ansContainer.off('click', '.remove-btn');
        $ansContainer.off('input change paste', '.textarea');
    }

    let updateHeightTextarea = function($textarea) {
        $textarea.height('auto');
        $textarea.height($textarea.prop('scrollHeight') - 8);
    }

    AnswerView.prototype.validAllTextarea = function(){
        let self = this;
        this.$form.find('.ans-container textarea').each((idx, ele) => {
            self.validTextAreaValue($(ele))
        });
    }

    AnswerView.prototype.validTextAreaValue = function($textarea){
        let $ansRow = $textarea.closest('.ans-row');

        if(!isValidateXML($textarea.val())){
            $ansRow.addClass('has-error');
            return;
        }

        let ansResult = answerParser($textarea.val());
        if (ansResult){
            $ansRow.removeClass('has-error');
        } else {
            $ansRow.addClass('has-error');
        }
    }

    let isValidateXML = (text) => {
        return !/[\x00-\x08\x0B\x0C\x0E-\x1F]/.test(text)
    }

    return AnswerView;

});
