requirejs.config({
  paths: {
    jquery: 'CodeBlockProgrammingInteraction/runtime/js/lib/jquery.min'
  }
});

define([
  'jquery',
  'CodeBlockProgrammingInteraction/creator/widget/config',
  'CodeBlockProgrammingInteraction/lib/beautify'

], function ($, config, beautify) {
  return {
    /**
     *
     * @param rectangleContainer 短冊のある場所 (DOM)
     * @param executableCode 実行用コードか
     * @returns {string}
     */
    createCode(rectangleContainer, executableCode=false, options) {
      if (!rectangleContainer) throw new Error('解答欄が見つかりません。');
      let code = "";

      for (const rectangle of rectangleContainer.children)
        code += this.rectangleToCode(rectangleContainer,rectangle, executableCode)
    
      return beautify.js_beautify(code);
    },
    rectangleToCode(rcon,rectangle, codeAddWhileCount = false) {
      const $rectangle=$(rectangle)
      const type = $rectangle.attr('data-rectangle-type') || "";
      const codeid = $rectangle.attr('data-rectangle-id') || "";
      const showVal =  $rectangle.attr('data-rectangle-func-name') || "";

      if (!type) return "";
      const baseCode = this.getBaseCode(type,codeAddWhileCount,codeid);

      //コード埋め込み
      let replaceCode = baseCode.replaceAll("#", $rectangle.attr('data-rectangle-exec-value')).replaceAll("¥¥", showVal)
      // replaceCode.replaceAll("¥¥", showVal)

      if (replaceCode.match(/\¥textField_([a-zA-Z0-9]{1,3})/)) {
        const inputValue = $rectangle.find('input').val();
        replaceCode = replaceCode.replace(/\¥textField_([a-zA-Z0-9]{1,3})/, inputValue)
      }
      if (replaceCode.match(/\¥selectBox_([a-zA-Z0-9]{1,3})/)) {
        const selectValue = $rectangle.find('select').val();
        replaceCode = replaceCode.replace(/\¥selectBox_([a-zA-Z0-9]{1,3})/, selectValue)
      }
      const selid=replaceCode.match(/\¥ref_selectBox_([a-zA-Z0-9]{1,3})/)
      if (selid) {
        const select=rcon.querySelector("[data-sb-id=\""+selid[1]+"\"]")
        if(select){
          const selectValue = select.value
          replaceCode = replaceCode.replace(/\¥ref_selectBox_([a-zA-Z0-9]{1,3})/, selectValue)
        }
      }
      const texid=replaceCode.match(/\¥ref_textField_([a-zA-Z0-9]{1,3})/)
      if (texid) {
        const input=rcon.querySelector("[data-tf-id=\""+texid[1]+"\"]")
        if(input){
          const inputValue = input.value
          replaceCode = replaceCode.replace(/\¥ref_textField_([a-zA-Z0-9]{1,3})/, inputValue)
        }
      }
      //子がいない場合
      if (!baseCode.includes('$'))
        return replaceCode;

      const transformCode = ['$', "%%"];
      //ネストの数だけfor
      $rectangle.children('.box-none').each((index, nest) => {
        let nestCode = "";
        for (const rectangle of nest.children)
          nestCode += this.rectangleToCode(rcon,rectangle)
        replaceCode = replaceCode.replace(transformCode[index], nestCode);
      })
      return replaceCode
    },
    getBaseCode(rectangleType,codeAddWhileCount = false,codeid) {
      switch (rectangleType) {
        case "while":
          return codeAddWhileCount ? `var __flag__${codeid} = 0;/* SYSTEM */ while(#){__flag__${codeid}++;/* SYSTEM */ if(__flag__${codeid}===${config.maxWhileNum}) throw -10;$}`: 'while(#){$}'
        case "for":
          return codeAddWhileCount ? `var __flag__${codeid} = 0;/* SYSTEM */ for(#){__flag__${codeid}++;/* SYSTEM */ if(__flag__${codeid}===${config.maxWhileNum}) throw -10;$}`: 'for(#){$}'
        case "if":
          return "if(#){$}"
        case "ifElse":
          return "if(#){$}else{%%}"
        case "println":
          return "printBlock(#); "
        case "print":
          return "print(#); "
        case "func":
          return 'function ¥¥(#){$}'
        default:
          return  '#';
      }
    },
  }
});

