requirejs.config({
    paths: {
        jquery: 'CodeBlockProgrammingInteraction/runtime/js/lib/jquery.min'
    }
});

define(['jquery'], ($) => {
    return {
        PrintType: {
            Print: "Print",
            PrintBlock: "PrintBlock",
            BreakLine: "BreakLine",
            BreakLineBlock: "BreakLineBlock"
        },
        getCodeBlock($container) {
            return $container.find('.right-container > .rectangle').not(('.right-container > .spacerectangle')).get().map((el) => {
                const getTansaku = (rectangleEl) => {
                    const nests = $(rectangleEl).children('.box-none').get();
                    if (!nests.length) return {
                        id: rectangleEl.getAttribute('data-rectangle-id'),
                        type: rectangleEl.getAttribute('data-rectangle-type')
                    }
                    return {
                        id: rectangleEl.getAttribute('data-rectangle-id'),
                        type: rectangleEl.getAttribute('data-rectangle-type'),
                        nests: nests.map((nest) => {
                            const values = []
                            for (const childRectangle of nest.children)
                                values.push(getTansaku(childRectangle))
                            return values
                        })
                    }
                }
                return getTansaku(el)
            })
        },

        getTextBox($container) {
            const textbox = []
            $container.find('.right-container .rectangle').each((i, el) => {
                const input = $(el).children('.show-value').find('input').get(0)
                const getTansaku = (rectangleEl) => {
                    const nests = $(rectangleEl).children('.box-none').get();
                    if (!nests.length)
                        return {
                            id: rectangleEl.getAttribute('data-rectangle-id'),
                            nests: nests.length ? nests.map((nest) => {
                                const values = []
                                nest.children.forEach((childRectangle) => {
                                    getTansaku(childRectangle)
                                })
                                return values
                            }) : undefined
                        }
                }
                if (input) {
                    const id = el.getAttribute('data-rectangle-show-value').match(/\¥textField_([a-zA-Z0-9]{1,3})/)[1]
                    const value = input.value
                    textbox.push({
                        [id]: value
                    })
                }
                return el.getAttribute('data-rectangle-id')
            });

            return textbox;
        },

        getDropdown($container) {
            const dropdown = []
            $container.find('.right-container .rectangle').each((i, el) => {
                const select = $(el).children('.show-value').find('select').get(0)
                const getTansaku = (rectangleEl) => {
                    const nests = $(rectangleEl).children('.box-none').get();
                    if (!nests.length)
                        return {
                            id: rectangleEl.getAttribute('data-rectangle-id'),
                            nests: nests.length ? nests.map((nest) => {
                                const values = []
                                nest.children.forEach((childRectangle) => {
                                    getTansaku(childRectangle)
                                })
                                return values
                            }) : undefined
                        }
                }

                if (select) {
                    const id = el.getAttribute('data-rectangle-show-value').match(/\¥selectBox_([a-zA-Z0-9]{1,3})\[.+?\]/)[1]
                    const value = select.value;
                    dropdown.push({
                        [id]: value
                    })
                }
                return el.getAttribute('data-rectangle-id')
            });

            return dropdown;
        },

        toBoolean(val) {
            return val === "true" || val === true;
        },
        updateHeightTextarea($textarea) {
            $textarea.height('auto');
            $textarea.height($textarea.prop('scrollHeight') - 8);
        },
        genAnsFormat(resultList){
            return resultList
                .filter((item) => item.type != this.PrintType.BreakLine)
                .map(item => {
                    if (item.type == this.PrintType.PrintBlock){
                        return this.updateValueByType(item.value, true);  //printBlock
                    }
                    if (item.type == this.PrintType.Print && Array.isArray(item.values)) {
                        if (item.values.length === 1) {
                            return this.updateValueByType(item.values[0], true)
                        }
                        return `ln(${item.values.map(subItem => this.updateValueByType(subItem, true)).join(",")})`; //print multiple
                    }
                    return '';
                }).join("\n");
        },
        genAnsDisplay(resultList){
            return resultList
                .filter((item) => item.type != this.PrintType.BreakLine)
                .map(item => {
                    if (item.type == this.PrintType.PrintBlock){
                        return this.updateValueByType(item.value)
                    }
                    if (item.type == this.PrintType.Print && Array.isArray(item.values)) {
                        if (item.values.length === 1) {
                            return this.updateValueByType(item.values[0])
                        }
                        return item.values.map(subItem => this.updateValueByType(subItem)).join("");
                    }
                    return '';
                })
                .join("\n");
        },
        updateValueByType(value, isAnsFormat = false) {
            //console.log("type", typeof value, value)
            switch (typeof value) {
                case "number":
                case "boolean":
                    return value
                case "string":
                    if (isAnsFormat){
                        value = value
                            .replace(/\\/g, '\\\\')
                            .replace(/\n/g, '\\n')
                            .replace(/\r/g, '\\r')
                            .replace(/\t/g, '\\t')
                            .replace(/\f/g, '\\f')
                            .replace(/\v/g, '\\v')
                            .replace(/\u0000/g, '\\x00'); // \0
                        // if (value == '\\'){
                        //     value = '\\\\';
                        // }
                    }
                    return `"${value.replace(/"/g, '\\"')}"`
                case "object":
                    return JSON.stringify(value)
                case "undefined":
                    return "undefined"
                default:
                    return value
            }
        }

    }
})
