requirejs.config({
    paths: {
        jquery: 'CodeBlockProgrammingInteraction/runtime/js/lib/jquery.min'
    }
});

define([
    'lodash',
    'qtiCustomInteractionContext',
    'jquery',
    'taoQtiItem/portableLib/OAT/util/event',
    'taoQtiItem/qtiCreator/model/mixin/editable',
    'taoQtiItem/qtiCreator/model/mixin/editableInteraction',
    'CodeBlockProgrammingInteraction/creator/Rectangle',
    'CodeBlockProgrammingInteraction/lib/Sortable',
    'CodeBlockProgrammingInteraction/lib/timeworker',
    'CodeBlockProgrammingInteraction/creator/widget/config',
    'CodeBlockProgrammingInteraction/runtime/js/Handler',
    'CodeBlockProgrammingInteraction/creator/widget/utils',
    'CodeBlockProgrammingInteraction/runtime/js/NotifyDialog',
], function (
    _,
    qtiCustomInteractionContext,
    $,
    event,
    editable,
    editableInteraction,
    Rectangle,
    Sortable, timeWorker, config, Handler, utils, NotifyDialog
) {
    'use strict';

    var liquidsInteraction = {
        /**
         ']jjghgh * Custom Interaction Hook API: id
         */
        id: -1,
        peintResult: '',
        /**
         * Custom Interaction Hook API: getTypeIdentifier
         * A unique identifier allowing custom interactions to be identified within an item.
         * @returns {String} The unique identifier of this PCI Hook implementation.
         */
        getTypeIdentifier: function getTypeIdentifier() {
            return 'CodeBlockProgrammingInteraction';
        },
        _currentResponse: {base: {string: "{}"}},
        /**
         * Render the PCI :
         * @param {String} id
         * @param {Node} dom
         * @param {Object} config - json
         */
        initialize: function initialize(id, dom, config, assetManager) {
            // Register the value for the 'id' attribute of this Custom Interaction Hook instance.
            // We consider in this proposal that the 'id' attribute

            this.id = id;
            this.dom = dom;
            this.config = config || {};
            window.dom = dom
            let self = this;
            // Tell the rendering engine that I am ready.
            // Please note that in this proposal, we consider the 'id' attribute to be part of the
            // Custom Interaction Hood API. The Global Context will then inspect the 'id' attribute
            // to know which instance of the PCI hook is requesting a service to be achieved.
            qtiCustomInteractionContext.notifyReady(this);

            //add method on(), off() and trigger() to the current object
            event.addEventMgr(this);

            this.mainHandler = new Handler($(dom), config);
            this.mainHandler.main();
            this.resultManager = this.mainHandler.getResultManager();

            this.on('changeAnsList', function (ansList) {
                config.options.ansList = ansList;
                self.resultManager.setExpectedAnsList(config.options.ansList);
                self.mainHandler.updateTranslateBtnState();
            });

            this.on('changeEditorTab', function (tabName) {
                self.mainHandler.updateEditorTab(tabName)
            });

            this.on('changeRepeatFlag', function (repeatFlag) {
                config.options.repeatFlag = repeatFlag;
                self.mainHandler.updateRepeatFlag(repeatFlag);
            });

            this.on('changeExecTime', function (execTime) {
                config.options.execTime = execTime;
            });

            this.on('changeHintFlag', function (hintFlag) {
                config.options.hintFlag = hintFlag;
                self.mainHandler.updateHintFlag(hintFlag);
            });

            this.on('changeHint', function (hint) {
                config.options.hint = hint;
            });

            this.on('triggerSaveAnswerStates', function () {
                self.mainHandler.saveAnswerStates();
            });

            this.on('changeTitle', function (titleHtml) {
                config.options.title = titleHtml;
            });

            this.mainHandler.addSortableForLeftContainer();
            this.mainHandler.addSortableForRightContainer();

            $(dom).on('click', '.right-container .rmv', function () {
                const $parent = $(this).parent();
                const $addPositionEl = $parent.next();
                const $appendContainer = $parent.parent();
                const appendEl = [];
                $parent.children('.box-none').each((_, nest) => {
                    appendEl.push(nest.children);
                })
                if ($addPositionEl.length) {
                    for (const el of appendEl)
                        $addPositionEl.before(el);
                } else {
                    for (const el of appendEl)
                        $appendContainer.append(el)
                }
                $parent.remove()
                // let rectid = $parent[0].getAttribute("data-rectangle-id")
                // checkSameLocked(rectid)
            })

            // ドロップダウンの広がり
            $(dom).on('change', '.form-rect-select', function () {
                const select_val = $(this).val();
                if ($(this).val().length > 13) {
                    const thiswidth = 130;
                    let next_pixel = thiswidth + (($(this).val().length - 8) * 8);
                    if (next_pixel > 200) {
                        next_pixel = 200;
                    }
                    $(this).css('width', next_pixel + 'px');
                } else {
                    $(this).css('width', '130px');
                }
                $(this).find('option').not(`[value="${select_val}"]`).removeAttr('selected');
                $(this).find('option[value="' + select_val + '"]').attr('selected', 'selected');
            });


        },
        /**
         * Programmatically set the response following the json schema described in
         * http://www.imsglobal.org/assessment/pciv1p0cf/imsPCIv1p0cf.html#_Toc353965343
         *
         * @param {Object} interaction
         * @param {Object} response
         */
        setResponse: function setResponse(response) {
            this._currentResponse = response;
        },
        /**
         * Get the response in the json format described in
         * http://www.imsglobal.org/assessment/pciv1p0cf/imsPCIv1p0cf.html#_Toc353965343
         *
         * @param {Object} interaction
         * @returns {Object}
         */
        getResponse: function getResponse() {
            let resultDataJson = this.resultManager.getDataJson()
            this._currentResponse = {
                base: {
                    string: resultDataJson
                }
            };
            return this._currentResponse;

        },
        /**
         * Remove the current response set in the interaction
         * The state may not be restored at this point.
         *
         * @param {Object} interaction
         */
        resetResponse: function resetResponse() {
            this._currentResponse = {base: {string: "{}"}};
        },
        /**
         * Reverse operation performed by render()
         * After this function is executed, only the inital naked markup remains
         * Event listeners are removed and the state and the response are reset
         *
         * @param {Object} interaction
         */
        destroy: function destroy() {
            this.mainHandler?.destroy();
        },
        /**
         * Restore the state of the interaction from the serializedState.
         *
         * @param {Object} interaction
         * @param {Object} serializedState - json format
         */
        setSerializedState: function setSerializedState(state) {
            //console.log(state)
            if (state?.questionState) {
                this.mainHandler.restoreQuestionState(state?.questionState);
            }
            if (state?.execDisplay) {
                $(this.dom).find('.exec-print').html(state?.execDisplay)
            }

            if (state?.resultDataJson) {
                this.resultManager.setDataJson(state.resultDataJson);
            }

            if (state && state.response) {
                this.setResponse(state.response);
            }
        },
        /**
         * Get the current state of the interaction as a string.
         * It enables saving the state for later usage.
         *
         * @param {Object} interaction
         * @returns {Object} json format
         */
        getSerializedState: function getSerializedState() {
            const $this = $(this.dom)
            $this.find('input').each(function (index, element) { //keep input value when next question
                $(element).attr("value", $(element).val())
            })

            const questionState = this.mainHandler.getCurrentState();
            const execDisplay = $this.find('.exec-print').html();
            const resultDataJson = this.resultManager.getDataJson();

            return {response: this.getResponse(), questionState, execDisplay, resultDataJson};
        }
    };

    _.extend(liquidsInteraction, editable);
    _.extend(liquidsInteraction, editableInteraction);
    qtiCustomInteractionContext.register(liquidsInteraction);
});