requirejs.config({
    paths: {
        jquery: 'CodeBlockProgrammingInteraction/runtime/js/lib/jquery.min'
    }
});

define([
    'jquery',
    'CodeBlockProgrammingInteraction/creator/Rectangle',
    'CodeBlockProgrammingInteraction/creator/ToCode',
    'CodeBlockProgrammingInteraction/creator/widget/utils',
    'CodeBlockProgrammingInteraction/runtime/js/ResultManager',
    'CodeBlockProgrammingInteraction/runtime/js/StateManager',
    'CodeBlockProgrammingInteraction/runtime/js/execCode',
    'CodeBlockProgrammingInteraction/runtime/js/createObserver',
    'CodeBlockProgrammingInteraction/runtime/js/NotifyDialog',
    'CodeBlockProgrammingInteraction/lib/Sortable',

], function ($,Rectangle, ToCode, utils, ResultManager, StateManager, execCode, createObserver, NotifyDialog, Sortable) {

    let Handler = function ($root, config) {
        this.$root = $root;
        this.config = config;
        this.isEditorMode = $root.closest('.item-editor-item').length > 0;
        this.editorTab = 'QUESTION';

        this.$questionTitle = $root.find('.question-title');
        this.$openJSPopupBtn = $root.find('.open-js-popup');
        this.$closeJSPopupBtn = $root.find('.close-js-popup');

        this.$openHintPopupBtn = $root.find('.hint-popup-btn');
        this.$closeHintPopupBtn = $root.find('.hint-popup-close');

        this.$execBtn = $root.find('.exec-btn');
        this.$resetBtn = $root.find('.reset-btn');
        this.$translateBtn = $root.find('.translate-ans-btn');

        this.$execResultPanel = $root.find('.exec-print');

        this.resultManager = new ResultManager();
        this.resultManager.init();
        this.resultManager.setExpectedAnsList(config.options.ansList);

        this.stateManager = new StateManager();
        this.stateManager.init($root, config);
    };

    Handler.prototype.main = function () {
        this.initView();
        this.addEvents();
    };

    Handler.prototype.destroy = function () {
        this.disconnectObserver();
    };

    Handler.prototype.initView = function () {
        if (this.isEditorMode){
            this.$openJSPopupBtn.show();
            this.$translateBtn.show();
            this.$resetBtn.hide();
            this.$execBtn.hide();
            this.updateTranslateBtnState();
        } else {
            this.$openJSPopupBtn.hide();
            this.$translateBtn.hide();
            this.$resetBtn.show();
            this.$execBtn.show();
        }
        this.initQuestionTitle();
        this.updateQuestionView();
        this.updateRepeatFlag(this.config.options.repeatFlag);
        this.updateHintFlag(this.config.options.hintFlag);
    }

    Handler.prototype.initQuestionTitle = function () {
        let titleHtml = this.config.options.title;
        if(titleHtml && titleHtml !== ''){
            this.$questionTitle.html(titleHtml);
        } else {
            this.$questionTitle.html('<p>解答欄に追加できる短冊型コードは、30個までです。</p>\n' +
                '<p id="alert-repeat">この問題では、短冊型コードを繰り返し利用することはできません。</p>\n' +
                '<p>繰り返し短冊型コードは５階層までしか利用できません。</p>\n' +
                '<p>条件分岐短冊型コードは５階層までしか利用できません。</p>')
        }
    }

    Handler.prototype.updateQuestionView = function () {
        this.stateManager.allowUpdateAnswerState = false;
        Rectangle.setRectangles(this.$root, this.stateManager.getQuestionState())
        this.addSortableForNestItems(this.$root.find('.right-container [data-nest="nest"]'));
    }

    Handler.prototype.updateAnswerView = function () {
        Rectangle.setRectangles(this.$root, this.stateManager.getAnswerState())
        this.addSortableForNestItems(this.$root.find('.right-container [data-nest="nest"]'));
    }

    Handler.prototype.getCurrentState = function () {
        return this.stateManager.getState();
    }

    Handler.prototype.restoreQuestionState = function (questionState) {
        Rectangle.setRectangles(this.$root, questionState)
        this.addSortableForNestItems(this.$root.find('.right-container [data-nest="nest"]'));
    }

    Handler.prototype.addSortableForLeftContainer = function () {
        new Sortable(this.$root.find('.left-container').get(0), {
            group: {
                name: 'shared',
                pull: 'clone',
                put: false,
            },
            filter: ".filter",
            animation: 150,
            onAdd: function (evt) {
                console.log(evt)
            }
        });
    }

    Handler.prototype.addSortableForRightContainer = function () {
        let self = this;
        let sortableOptions = {
            group: 'shared',
            invertSwap: true,
            filter: ".filter",
            onAdd: function (evt) {
                if ($(evt.item).closest('.edit-answer, .runtime').length > 0) {
                    $(evt.item).prevUntil('[class*="box-"]').remove();
                    $(evt.item).nextUntil('[class*="box-"]').remove();
                }

                if ($(document).find('.right-container .rectangle').length > 30) {
                    $(evt.item).remove();
                    NotifyDialog.showAlert($(dom),'解答欄に追加できる短冊型コードは、30個までです。');
                    return false;
                }
                if (!self.AllowRepeatItem()) {
                    self.RemoveIfExist(evt.item);
                }

                self.addSortableForNestItems($(evt.item));
            },
            onFilter: function (evt) {
            },
            animation: 100,
        }

        Sortable.create(this.$root.find('.right-container').get(0), sortableOptions);
        this.addSortableForNestItems(this.$root.find('.right-container [data-nest="nest"]'));
    }

    Handler.prototype.addSortableForNestItems = function ($parent) {
        let self = this;
        $parent.find('.box-none').each((_, el) => {
            self.addSortable(el);
        });
    }

    Handler.prototype.addSortable = function (item) {
        let self = this;
        let sortableOptions = {
            group: 'shared',
            invertSwap: true,
            filter: ".filter",
            onAdd: function (evt) {
                if (!self.AllowRepeatItem()) {
                    self.RemoveIfExist(evt.item);
                }
                self.addSortableForNestItems($(evt.item));
            },
            onFilter: function (evt) {
            },
            animation: 100,
        }
        Sortable.create(item, sortableOptions)
    }

    Handler.prototype.RemoveIfExist = function (item) {
        const id = item.getAttribute('data-rectangle-id')
        const rightTargets = this.$root.find('.right-container').find(`[data-rectangle-id=${id}]`).get()

        for (let el of rightTargets) {
            if (el === item) continue;
            item.remove();
            return;
        }
    }

    Handler.prototype.AllowRepeatItem = function () {
        return utils.toBoolean(this.config.options.repeatFlag);
    }

    Handler.prototype.addEvents = function () {
        let self = this;
        if (this.isEditorMode){
            this.$openJSPopupBtn.on('click', () => this.openJSPopup());
            this.$closeJSPopupBtn.on('click', () => this.closeJSPopup());

            this.$translateBtn.on('click', () => this.onTranslateBtnClick());
        }

        this.$openHintPopupBtn.on('click', () => this.openHintPopup());
        this.$closeHintPopupBtn.on('click', () => this.closeHintPopup());

        this.$execBtn.on('click', () => this.executeCode());
        this.$resetBtn.on('click', () => this.onResetClick());
    };

    Handler.prototype.updateEditorTab = function (tabName) {
        this.editorTab = tabName;

        if (this.isQuestionMode()){
            this.$execBtn.hide();
            this.$resetBtn.hide();
            this.updateQuestionView();
        } else if (this.isAnswerMode()){
            this.$execBtn.show();
            this.$resetBtn.show();
            this.updateAnswerView();
        }

        this.updateTranslateBtnState();
        this.disconnectObserver();
        this.createObserver();
    }

    Handler.prototype.updateTranslateBtnState = function () {
        if (this.isEditorMode && this.resultManager.hasExecResult() && !this.isMaxAnswerRow()) {
            this.$translateBtn.removeClass('disabled')
        } else {
            this.$translateBtn.addClass('disabled')
        }
    }

    Handler.prototype.openJSPopup = function () {
        const code = ToCode.createCode(this.$root.find('.right-container').get(0));
        this.$root.find('.printJsCode').text(code);
        this.$root.find('.js-popup').addClass('is-show');
    }

    Handler.prototype.closeJSPopup = function () {
        this.$root.find('.js-popup').removeClass('is-show');
    }

    Handler.prototype.openHintPopup = function () {
        let hint = this.config.options.hint;
        if (hint) {
            const hintLines = hint.split("\n").map((str, index) => {
                return `<div><span>${str}</span></div>`
            });
            this.$root.find('.hint-area').html(hintLines);
        }
        this.$root.find('.hint-popup').addClass('is-show');
    }

    Handler.prototype.closeHintPopup = function () {
        this.$root.find('.hint-popup').removeClass('is-show');
    }

    Handler.prototype.executeCode = function () {
        let self = this;
        let code = this.getCode();
        this.beforeExecuteCode();

        let execOptions = {
            $container: this.$root,
            timeout: this.config.options.execTime * 1000,
            onInvalid: (msg) => {
                this.afterExecuteCode();
                NotifyDialog.showAlert(this.$root, msg);
            },
            onTimeout: () => {
                this.afterExecuteCode();
                NotifyDialog.showAlert(this.$root,'プログラムが長時間実行されたためプログラムを停止しました。プログラムに問題がないか確認のうえ再度実行してください。');
            },
            onCompleted: (resultList) => {
                this.afterExecuteCode();

                if (resultList.error) {
                    this.$execResultPanel.text(resultList.error);
                } else {
                    this.showPrintResult(resultList);
                }

                this.resultManager.addExecLog(
                    code,
                    resultList,
                    utils.getCodeBlock(this.$root),
                    utils.getTextBox(this.$root),
                    utils.getDropdown(this.$root)
                );
                this.updateTranslateBtnState();
            }
        }

        //console.log("code:", code)
        execCode(code, execOptions)
    }

    Handler.prototype.isMaxAnswerRow = function(){
        return Array.isArray(this.config.options.ansList) && this.config.options.ansList.length >= 10;
    }

    Handler.prototype.showPrintResult = function (resultList) {
        let $pre = $('<pre></pre>').text(utils.genAnsDisplay(resultList))
        this.$execResultPanel
            .append($pre)
            .append('<p class="result-complete">プログラムの実行が完了しました。</p>');
    }

    Handler.prototype.beforeExecuteCode = function () {
        this.$execResultPanel.text('待っている........')
        this.$execBtn.css("pointer-events", "none");
        this.resultManager.resetExecResult();
        this.updateTranslateBtnState();
    }

    Handler.prototype.afterExecuteCode = function () {
        this.$execResultPanel.empty();
        this.$execBtn.css("pointer-events", "auto");
    }

    Handler.prototype.onResetClick = function () {
        NotifyDialog.showConfirm(this.$root, '解答欄を元の状態に戻します。よろしいですか？', (isOk) => {
            if (isOk) {
                this.updateQuestionView();
                this.resultManager.resetExecResult();
                this.updateTranslateBtnState();
                this.$execResultPanel.empty();
            }
        })
    }

    Handler.prototype.onTranslateBtnClick = function () {
        if (!this.resultManager.hasExecResult()) return;
        let ansFormat = utils.genAnsFormat(this.resultManager.execResult);

        if (this.isQuestionMode()){
            this.triggerAddAnswer(ansFormat);
        } else if (this.isAnswerMode()){
            this.updateAnswerList(ansFormat);
            $('.link[data-state="question"]').trigger('click');
        }

        this.resultManager.resetExecResult();
        this.updateTranslateBtnState();
    }

    Handler.prototype.triggerAddAnswer = function (ansFormat) {
        this.$root.trigger($.Event('triggerAddAnswer', { detail: ansFormat }));
    }

    Handler.prototype.updateAnswerList = function (ansFormat) {
        let ansListTmp = this.config.options.ansList.filter(value => value !== '');
        ansListTmp.push(ansFormat);

        if (ansListTmp.length == 1) {
            ansListTmp.unshift('');
        }
        this.config.options.ansList = ansListTmp;
    }

    Handler.prototype.getCode = function () {
        return ToCode.createCode(this.$root.find('.right-container').get(0), true);
    }

    Handler.prototype.updateHintFlag = function (hintFlag) {
        if (utils.toBoolean(hintFlag)) {
            this.$openHintPopupBtn.show();
        } else {
            this.$openHintPopupBtn.hide();
        }
    }

    Handler.prototype.updateRepeatFlag = function (repeatFlag) {
        if (utils.toBoolean(repeatFlag)) {
            this.$root.find('#alert-repeat').hide();
        } else {
            this.$root.find('#alert-repeat').show();
        }
    }

    Handler.prototype.disconnectObserver = function () {
        this.observer?.disconnect();
    }

    Handler.prototype.createObserver = function () {
        let attributeFilter = [
            "data-rectangle-type",
            "data-rectangle-show-value",
            "data-rectangle-func-name",
            "data-rectangle-exec-value",
            "data-rectangle-id",
            "data-rectangle-arrowinput-value",
            // "value",
            // "selected"
        ];
        if (this.isAnswerMode()){
            attributeFilter = attributeFilter.concat([
                "value",
                "selected"
            ])
        }

        this.observer = createObserver(this.$root,
            {
                attributeFilter: attributeFilter,
                mutationCallback: (records) => this.mutationCallback(records)
            }
        )
    }

    Handler.prototype.mutationCallback = function (records) {
        if (this.isQuestionMode()){
            this.stateManager.allowUpdateAnswerState = true;
            this.stateManager.saveQuestionStates();
        } else if (this.isAnswerMode()){
            this.stateManager.saveAnswerStates();
        }
    }

    Handler.prototype.saveAnswerStates = function () {
        this.stateManager.saveAnswerStates();
    }

    Handler.prototype.isQuestionMode = function () {
        return this.editorTab === "QUESTION";
    }

    Handler.prototype.isAnswerMode = function () {
        return this.editorTab === "RESPONSE";
    }

    Handler.prototype.getResultManager = function () {
        return this.resultManager;
    }


    return Handler;
});