requirejs.config({
    paths: {
        jquery: 'CodeBlockProgrammingInteraction/runtime/js/lib/jquery.min'
    }
});

define([
    'jquery',
    'lodash',
    'CodeBlockProgrammingInteraction/runtime/js/answerParser',
    'CodeBlockProgrammingInteraction/creator/widget/utils',

], function ($, _, answerParser, utils) {

    let ResultManager = function () {
        this.init();
    };

    ResultManager.prototype = {
        init: function () {
            this.data = {
                startings: [new Date().getTime()],
                finalAnswer: null,
                submitCount: 0,
                codes: [],
            };
            this.expectedAnsList = null;
            this.$cppResponseHolder = $('.cbp-score .cbp-result-holder');
            this.execResult = null;
        },
        setExpectedAnsList: function (expectedAnsList) {
            this.expectedAnsList = Array.isArray(expectedAnsList) ? expectedAnsList : [expectedAnsList];
        },
        setData: function (data) {
            this.data = data;
        },
        getData: function () {
            return this.data;
        },
        setDataJson: function (dataJson) {
            this.data = JSON.parse(dataJson);
            this.data.startings.push(new Date().getTime())
        },
        getDataJson: function () {
            return JSON.stringify(this.data);
        },
        hasExecResult: function () {
            return this.execResult && !this.execResult.error && Array.isArray(this.execResult);
        },
        resetExecResult: function () {
            this.execResult = null;
        },

        addExecLog: function (code, execResult, codeblock, textbox, dropdown) {
            let date = new Date();
            let codeObj = {
                code: code,
                execTime: formatTime(date),
                execUnixTime: date.getTime(),
                result: '',
                codeblock: codeblock,
                textbox: textbox,
                dropdown: dropdown,
            };

            if (execResult.error) { //is error case
                this.$cppResponseHolder.val('ERROR')
                this.data.finalAnswer = null;
            } else { //is normal case
                let ansDisplay = utils.genAnsDisplay(execResult);
                codeObj.result = ansDisplay;
                this.data.finalAnswer = ansDisplay;
                this.$cppResponseHolder.val(this.judgeResult(execResult))
            }

            this.data.codes.push(codeObj);
            this.data.submitCount++;
            this.execResult = execResult;
        },
        judgeResult: function (execResult, expectedAnsList = null) {
            if (expectedAnsList) {
                this.expectedAnsList = expectedAnsList;
            }
            if (!this.expectedAnsList
                || !Array.isArray(this.expectedAnsList)
                || !this.expectedAnsList.length
                || (this.expectedAnsList.length === 1 && this.expectedAnsList[0] === '')
            ) {
                return "EXPECT ANSWER IS UNDEFINED";
            }

            let ansFormat = utils.genAnsFormat(execResult);
            let currentAnsParser = answerParser(ansFormat);

            if (!execResult || execResult === '') {
                return "CURRENT ANSWER IS UNDEFINED";
            }

            let resultMsg = 'WRONG ANSWER'
            try {
                for (let expectedAns of this.expectedAnsList) {
                    let expectedAnsParser = answerParser(expectedAns);
                    if (!expectedAnsParser || (Array.isArray(expectedAnsParser) && expectedAnsParser.length === 0)) {
                        continue;
                    }

                    //console.log("COMPARE RST:", currentAnsParser, expectedAnsParser);
                    if (_.isEqual(currentAnsParser, expectedAnsParser)) {
                        resultMsg = "PASS"
                        break;
                    }
                }
            } catch (e) {
                //console.log("ERROR:", e);
                return "EXPECT ANSWER IS UNDEFINED";
            }

            return resultMsg
        },
    };

    //2024-06-18 15:33:03.239
    let formatTime = function (date) {
        let formattedDate = date.toLocaleString('ja-JP', {
            year: 'numeric',
            month: '2-digit',
            day: '2-digit',
            hour: '2-digit',
            minute: '2-digit',
            second: '2-digit',
            hour12: false
        }).replace(/\//g, '-'); // Replace '/' with '-' to match the desired format

        let milliseconds = String(date.getMilliseconds()).padStart(3, '0');
        return `${formattedDate}.${milliseconds}`
    }

    return ResultManager;
});