define([], function () {

    let execute = (code) => {
        try {
            let PrintType = {
                Print: "Print",
                PrintBlock: "PrintBlock",
                BreakLine: "BreakLine",
                BreakLineBlock: "BreakLineBlock"
            };

            const resultList = [];
            let beforeType = null;

            const printTarget = function (target){
                if (typeof target === "object") {
                    try {
                        return JSON.parse(JSON.stringify(target));
                    } catch {
                        throw Error('表示出来ないオブジェクトです。');
                    }
                }
                return target;
            }

            const printBlock = function(target) {
                if (arguments.length == 0){
                    if (beforeType == PrintType.Print){
                        resultList.push({type: PrintType.BreakLine, value : ""});
                        beforeType = PrintType.BreakLine;
                    } else if (beforeType == PrintType.PrintBlock
                        || beforeType == PrintType.BreakLine
                        || beforeType == PrintType.BreakLineBlock
                    ){
                        resultList.push({type: PrintType.BreakLineBlock, value : ""});
                        beforeType = PrintType.BreakLineBlock;
                    }

                    return;
                }

                let value = printTarget(target);
                resultList.push({type: PrintType.PrintBlock, value : value});
                beforeType = PrintType.PrintBlock;
            };

            const print = function(target){
                if (arguments.length != 1) throw Error('The print() method accepts only one parameter');

                let lastItem = resultList[resultList.length - 1];
                let value = printTarget(target);

                if (lastItem && lastItem.type == PrintType.Print) {
                    lastItem.values.push(value);//append to last print arr
                } else {
                    resultList.push({type: PrintType.Print, values : [value]} ); //new print arr
                }

                beforeType = PrintType.Print;
            };

            const codeFunction = new Function('printBlock', 'print', code)
            codeFunction(printBlock, print);

            return resultList;
        } catch (e) {
            let error;
            console.error("execute code error:", e);
            switch (e === -10 ? e : e.constructor) {
                case -10: {
                    error = '規定回数以上の繰り返し処理が実行されたためプログラムを停止しました。プログラムに問題がないか確認のうえ再度実行してください。';
                    break;
                }
                case TypeError || ReferenceError: {
                    error = 'エラーが発生しました。変数の利用に誤りがないか確認してください。';
                    break;
                }
                case SyntaxError || RangeError: {
                    error = 'エラーが発生しました。構文に誤りがないか確認してください。';
                    break;
                }
                default: {
                    error = 'エラーが発生しました。プログラムを再度確認してください。';
                }
            }
            return {
                error : error
            };
        }

    }

    return function createCodeWorker() {
        let blobURL = URL.createObjectURL(new Blob([
            '(',
            function(execute) {
                self.addEventListener('message', function(code) {
                    const result = execute(code.data)
                    self.postMessage({ result: result })
                }, false)
            }.toString(),
            ')(',
            execute.toString(),
            ')'
        ], { type: 'application/javascript' }));

        return new Worker(blobURL);
    }
});