requirejs.config({
    paths: {
        jquery: 'CodeBlockProgrammingInteraction/runtime/js/lib/jquery.min'
    }
});

define([
    'jquery',
    'CodeBlockProgrammingInteraction/runtime/js/createCodeWorker',
    'CodeBlockProgrammingInteraction/creator/widget/utils'

], function ($, createCodeWorker, utils) {

    let vaidate = ($container) => {
        //解答欄に何も設定されてないときのエラー
        const result_element = $container.find('.right-container');
        const input_items = result_element[0].querySelectorAll('.form-rect');
        const select_items = result_element[0].querySelectorAll('.form-rect-select');
        const nest_items = result_element[0].querySelectorAll("[data-nest]")

        let empty_input = [];
        let empty_select = [];
        let empty_arrowitem = 0;
        let nets_overlimit = 0;

        //未入力のinputがないかチェック
        Array.from(input_items).forEach((item) => {
            const val = item.value;
            if (!val) {
                empty_input.push(item);
            }

            //入力許可ワードのチェック
            const validation_value = item.parentNode.parentNode.getAttribute('data-rectangle-arrowinput-value')
            if (validation_value) {
                const validation_value_array = validation_value.split(',');
                for (let i = 0; i < val.length; i++) {
                    if (validation_value_array.indexOf(val.charAt(i)) > -1) {
                    } else {
                        empty_arrowitem = 1;
                    }
                }
            }
        });

        //未選択のドロップダウンがないかチェック
        Array.from(select_items).forEach((item) => {
            const val = item.value;
            if (!val) {
                empty_select.push(item);
            }
        });

        Array.from(nest_items).forEach((item) => {
            if (item.querySelectorAll('[data-nest="nest"] > .box-none > [data-nest="nest"]:first-child').length > 4) {
                nets_overlimit = 1;
            }
        });

        //警告判定
        const warn = $container.find('.block-alert-border')[0];
        if (warn) {
            return "短冊型コードの編集が完了していません。短冊型コードに適切な値を入力して再度実行してください。";
        }
        if (empty_input.length) {
            return "未入力のテキストボックスがあります。テキストボックスを入力後に再度実行してください。";
        }
        if (empty_select.length) {
            return "未選択のドロップダウンリストがあります。ドロップダウンリストから選択後に再度実行してください。";
        }
        if (empty_arrowitem === 1) {
            return "テキストボックスに利用できない文字列が含まれています。テキストボックスの入力内容を見直して再度実行してください。";
        }
        if (!result_element.html()) {
            return '解答欄に短冊型コードがありません。短冊型コードを解答欄に追加して再度実行してください。';
        }
        if (nets_overlimit === 1) {
            return '繰り返し短冊型コードは５階層までしか利用できません。';
        }
        return null;
    }

    return function execCode(code, options, hasValidate = true) {
        if (hasValidate) {
            let msg = vaidate(options.$container);
            if (msg) {
                options?.onInvalid(msg);
                return;
            }
        }

        let timeout = options.timeout;
        let finish = false;
        let worker = createCodeWorker();

        worker.postMessage(code);
        worker.addEventListener('message', function(event) {
            finish = true;
            options?.onCompleted(event.data.result);
        }, false);

        setTimeout(function() {
            if(!finish){
                worker.terminate();
                options?.onTimeout();
            }
        }, timeout);

    }


});

